'use strict';

var ParserHelpers = require('webpack/lib/ParserHelpers');
var OpenUI5LazyInstanceDependency = require('./OpenUI5LazyInstanceDependency');
var OpenUI5RequireItemDependency = require('./OpenUI5RequireItemDependency');
var OpenUI5RequireContextDependency = require('./OpenUI5RequireContextDependency');
var ContextDependencyHelpers = require('./ContextDependencyHelpers');

class OpenUI5RequireDependencyParserPlugin {
  constructor(options) {
    this.options = options;
  }

  apply(parser) {
    var options = this.options || {};
    var globals = {
      'sap.ui.core.theming.Parameters': 'sap/ui/core/theming/Parameters',
      'sap.ui.core.LabelEnablement': 'sap/ui/core/LabelEnablement',
      'sap.ui.core.HTML': 'sap/ui/core/HTML',
      'sap.m.CheckBox': 'sap/m/CheckBox',
      'sap.viz.ui5.theming.Util': 'sap/viz/ui5/theming/Util',
      'sap.ui.unified.DateRange': 'sap/ui/unified/DateRange'
    };

    parser.plugin('call sap.ui.require', function (expr) {
      var param = parser.evaluateExpression(expr.arguments[0]);
      if (expr.arguments[0].type === 'Literal') {
        parser.applyPluginsBailResult('call require:openui5:item', expr, param);
        return true;
      } else if (expr.arguments[0].type === 'ArrayExpression') {
        parser.applyPluginsBailResult('call require:openui5:array', expr, param);
        return true;
      }
      return false;
    });

    parser.plugin('call sap.ui.requireSync', function (expr) {
      var param = parser.evaluateExpression(expr.arguments[0]);

      var result = parser.applyPluginsBailResult('call require:openui5:item', expr, param);
      if (!result) {
        result = parser.applyPluginsBailResult('call require:openui5:context', expr, param);
      }
      return result;
    });

    parser.plugin('call lazyInstanceof', function (expr) {
      var param1 = parser.evaluateExpression(expr.arguments[1]);
      if (param1.string !== 'sap/ui/app/Application') {
        var dep = new OpenUI5LazyInstanceDependency(param1.string, expr.arguments[0].name, expr.range);
        dep.loc = expr.loc;
        dep.optional = !!parser.scope.inTry;
        parser.state.current.addDependency(dep);
        return true;
      }
      return false;
    });

    parser.plugin('call jQuery.sap.require', function (expr) {
      var param = parser.evaluateExpression(expr.arguments[0]);
      if (expr.arguments[0].type === 'Literal') {
        parser.applyPluginsBailResult('call require:openui5:item', expr, param);
        return true;
      }
      parser.applyPluginsBailResult('call require:openui5:remove', expr, param);
      return false;
    });

    parser.plugin('call $.sap.require', function (expr) {
      var param = parser.evaluateExpression(expr.arguments[0]);
      if (expr.arguments[0].type === 'Literal') {
        parser.applyPluginsBailResult('call require:openui5:item', expr, param);
        return true;
      }
      return false;
    });

    parser.plugin('call require:openui5:array', function (expr, param) {
      param.items.forEach(function (param) {
        parser.applyPluginsBailResult('call require:openui5:item', expr, param);
      });
      return true;
    });

    parser.plugin('call require:openui5:item', function (expr, param) {
      if (param.isString()) {
        var item = param.string;
        if (item.substr(0, 10) !== 'jquery.sap' && item.substr(0, 2) !== './') {
          item = item.replace(/\./g, '/');
        }
        var dep = new OpenUI5RequireItemDependency(item, expr.range);
        dep.loc = expr.loc;
        dep.optional = !!parser.scope.inTry;
        parser.state.current.addDependency(dep);
        return true;
      }
      return false;
    });

    parser.plugin('call require:openui5:context', function (expr, param) {
      var dep = ContextDependencyHelpers.create(OpenUI5RequireContextDependency, expr.range, param, expr, options);
      if (!dep) return false;
      dep.loc = expr.loc;
      dep.optional = !!parser.scope.inTry;
      parser.state.current.addDependency(dep);
      return true;
    });

    parser.plugin('call require:openui5:remove', ParserHelpers.toConstantDependency(process.env.NODE_ENV === 'production' ? '' : 'console.warn("UI5 tried to dynamically require a module. If it doesn\'t exist in the bundle a error might follow.")'));

    parser.plugin('expression require:openui5:global', function (expr, request) {
      var dep = new OpenUI5RequireItemDependency(request, expr.range, true);
      dep.loc = expr.loc;
      dep.optional = !!parser.scope.inTry;
      parser.state.current.addDependency(dep);
      return true;
    });

    Object.keys(globals).forEach(function (key) {
      parser.plugin(`expression ${key}`, function (expr) {
        return parser.applyPluginsBailResult('expression require:openui5:global', expr, globals[key]);
      });
    });
  }
}
module.exports = OpenUI5RequireDependencyParserPlugin;